# Copyright (c) 2018 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.

from __future__ import absolute_import, division, print_function
import os.path
import re

# A list of products that have ASU support enabled on the shutdown path only, not
# on the bootup path.
products = []

def isAsuBootupUnsupported():
   # Check /etc/prefdl to see if the SID is in the list of products where the next
   # images does not support Asu bootup path
   preFdlPath = '/etc/prefdl'
   if os.path.isfile( preFdlPath ):
      with open( preFdlPath ) as prefdl:
         prefdlStr = prefdl.read()
         sidMatch = re.search( r'^SID:\s*(.+)\s*$', prefdlStr,
                               re.MULTILINE | re.IGNORECASE )

         if sidMatch:
            sid = sidMatch.group( 1 ).lower()
            return any( sid.startswith( s ) for s in products )

   # Return false if prefdl information is missing
   return False

def checkAsuSupport( ctx ):
   '''Determines if the next image supports Asu (SSU) reload bootup path. Mostly,
   both Asu shutdown and bootup path are supported or not supported at the same time
   but in some cases when Asu shudown path is supported but the next image does not
   support the Asu bootup path, the Asu reload Cli is available to test the Asu
   shutdown path but the Asu reload will not succeed, so we need to block it when the
   command is issued by the user in a non-testing environment.
   See AID 8350 for more details.
   '''
   errMsg = "SSU reload from %s to %s is not supported on the current platform." % (
               ctx.currentVersion.version(), ctx.nextVersion.version() )

   if isAsuBootupUnsupported():
      ctx.addError( errMsg )
      return False

   return True

def Plugin( ctx ):
   category = [ 'ASU' ]
   ctx.addPolicy( checkAsuSupport, category )
