# Copyright (c) 2020 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.

'''
This ReloadPolicyPlugin prevents EOS 4.24 and later images from being loaded on Alta
systems since they no longer support those platforms.
'''

import re


# Where to find the prefdl and how to find the SID in its contents.
prefdlPath = '/etc/prefdl'
sidRegex = r'^SID:\s*(.+)\s*$'


# List of SIDs for Alta systems.
altaSids = [ 'Cotati', 'Rosa', 'SantaRosa' ]

# Error message to let the user know that the platform is deprecated.
errorMessageTemplate = '''
The last supported version for this platform is EOS 4.23, but this image is EOS
{nextVersion}.
'''.strip()


def isAltaSystem():
   # Read the prefdl to determine if this is an Alta system.
   try:
      with open( prefdlPath ) as prefdl:
         prefdlContents = prefdl.read()
   except IOError:
      return False

   # Extract the SID from the contents of the prefdl.
   sidMatch = re.search( sidRegex, prefdlContents, re.MULTILINE | re.IGNORECASE )

   # Can't find the SID; assume this is not an Alta system.
   if not sidMatch:
      return False

   # Check if we know this SID.
   sid = sidMatch.group( 1 ).lower()
   return any( sid.startswith( s.lower() ) for s in altaSids )


def deprecatedAltaSystems( ctx ):
   # No need to check the EOS version being loaded. The simple fact that this plugin
   # is present and running means that Alta has been deprecated, so we already know
   # that we don't support this system.

   nextVersion = ctx.nextVersion.version()
   errorMessage = errorMessageTemplate.format( nextVersion=nextVersion )

   ctx.addError( errorMessage )

   return False


def Plugin( ctx ):
   categories = [ 'ASU', 'ASU+', 'General' ]

   if isAltaSystem():
      ctx.addPolicy( deprecatedAltaSystems, categories )
