# Copyright (c) 2020 Arista Networks, Inc.  All rights reserved.
# Arista Networks, Inc. Confidential and Proprietary.

'''
This ReloadPolicyPlugin generates a warning when the user attempts to load EOS 4.24
or later images when there are Arad- or FE1600-based cards present in the system,
since these images no longer support those cards.
'''

import re


# Warning message to let the user know that the platform is deprecated.
warningMessageTemplate = '''
7500E Series modules are not supported by this image (EOS {nextVersion}) and will \
not operate after the switch reloads; please use EOS 4.23 or earlier if you wish to \
use these modules.
'''.strip()


def systemHasAradOrFe1600Cards( ctx ):
   def isAradOrFe1600Card( modelName ):
      # No pylint, I don't think a ternary condition would be more readable here.
      # pylint: disable=consider-using-ternary
      return ( '7500E' in modelName and 'SUP' not in modelName ) \
             or re.search( '750[48]E-FM', modelName )

   try:
      import LazyMount
      entityMib = LazyMount.mount( ctx.mode.entityManager, 'hardware/entmib',
                                   'EntityMib::Status', 'r' )

      return any( isAradOrFe1600Card( cardSlot.card.modelName )
                  for cardSlot in entityMib.chassis.cardSlot.itervalues()
                  if cardSlot.card )
   except: # pylint: disable=bare-except
      # Yes pylint, I *do* want to catch all possible exceptions here.
      # If anything goes wrong, just assume there are no Arad/FE1600 cards in the
      # system.
      return False


def deprecatedAradOrFe1600Cards( ctx ):
   # No need to check the EOS version being loaded. The simple fact that this plugin
   # is present and running means that Arad has been deprecated, so we already know
   # that we don't support these cards.

   nextVersion = ctx.nextVersion.version()
   warningMessage = warningMessageTemplate.format( nextVersion=nextVersion )

   # Don't prevent the system from being reloaded; just issue a warning.
   ctx.addWarning( warningMessage )
   return True


def Plugin( ctx ):
   categories = [ 'ASU', 'ASU+', 'General' ]

   if systemHasAradOrFe1600Cards( ctx ):
      ctx.addPolicy( deprecatedAradOrFe1600Cards, categories )
